var express = require('express')
var webpack = require('webpack')
var fs = require('fs')
var path = require('path')
var session = require('express-session')
var bodyParser = require('body-parser')
var webpackConfig = require('../webpack.config.js')
/**initialize */
var app = express() 
var compiler = webpack(webpackConfig)//编译webpack配置文件
/**use express session */
app.use(bodyParser.urlencoded({ extended: false }))
/**set our webpack middlewire*/
var devMiddleware = require('webpack-dev-middleware')(compiler,
	{
		publicPath: webpackConfig.output.publicPath,
		stats: {
			colors: true,
			chunks: false
		}
	})
/**use express session*/
app.use(session({
	resave: true,
	saveUninitialized: true,
	secret: 'keyboard cat',
	cookie: { maxAge: 500000 }
})) 
/**read all apis and registry all*/
var apiDirName = path.resolve(__dirname,'./api') 
var fsArr = fs.readdirSync(apiDirName)
fsArr.forEach(function (item) {//注册api
	if(item != 'README.MD' && item != 'allPath.js'){
		try {
			var api = require(path.resolve(apiDirName,item))
			for(var key in api){
				var thisControl = api[key]
				app[thisControl.method].apply(app, [thisControl.path].concat(thisControl.cb))
			}
		} catch (error) {
			/*eslint-disable*/
			console.error('%s get API failed!',item)
			/*eslint-enable*/
		}
	}
})
/**set our HMR middlewire*/
var hotPublish = require('webpack-hot-middleware')(compiler,{
    log: () => {}
})
/**regist our reload event*/
compiler.plugin('compilation', function (compilation) {
	compilation.plugin('html-webpack-plugin-after-emit', function (data, cb) {
		// publish event
		hotPublish.publish({ action: 'reload' })
		cb()
	})
})

app.use(devMiddleware)
app.use(hotPublish)

/**
 * start our server
 */
var server = app.listen(8886, function () {
	/*eslint-disable */ 
    var host = server.address().address;
    var port = server.address().port;
    console.log('Example app listening at http://%s:%s', host, port);
    /*eslint-enable */
})
