/**
 * The scrollbar based on react
 * wangyi. 2017.12.1
 */
import * as React from "react";
import { Event } from "_debugger";

const wrapWrapStyle:React.CSSProperties = {
    height:'100%',
    overflow:'hidden',
}
const scrollbarBaseStyle:React.CSSProperties = {
    position: 'absolute',
    top: '0px',
    transition: 'opacity .5s',
    cursor: 'default',
    zIndex:99999
}
/****************************************************************************** */
/****************TS Components property declaration bugin********************** */
interface IPWrapWrapComponents{
    onMouseEnter():void;
    onmouseout():void;
}
interface IPWrapComponents {
    getBarHeight():void
}
interface IPScrollBar {
    
    options:{
        width?:string;
        background?:string;
        right?:string;
        deltaY?:number;
        [key:string]:any;
        float?:string;
    }
    /**
     * 
     * 
     * @param {Object} [bar] The information about ScrollBar
     * @param {Object} [content] The information about the scrolled content
     * @memberof IScrollBar
     */
    methods?:{
        scroll(bar?:Object,content?:Object):void        
    }
    height:number;
    top:number;
}
interface ISScrollBar {
   
    scrollInnerHeight:string;
    minBarHeight:number;
}
interface IPScrollContainer{
    options?:{
        width?:string;
        background?:string;
        right?:string;
        deltaY?:number;
        [key:string]:any;
    }
    methods?:{
        scroll(bar?:Object,content?:Object):void        
    }
}
interface ISScrollContainer{
    mousedown:boolean,
    innerdeltaY:number;
    height:number;
    top:number;
    scrollInnerHeight:number;
    scrollHeight:number;
}
/****************TS Components property declaration end************************ */
/****************************************************************************** */

//The component that wraps the components that waprs the component
class WrapWrapComponents extends React.Component<IPWrapWrapComponents>{
    render(){
        const children = this.props.children;
        return (
            <div 
            style={wrapWrapStyle} ref="WrapWrapComponents"
            onMouseEnter={this.props.onMouseEnter}
            onMouseOut={this.props.onmouseout}
            >
                {children}
            </div>
        );
    }
}


//The component wraps the component
class WrapComponents extends React.Component<IPWrapComponents>{
    render(){
        const children = this.props.children;
        return (
            <div 
            onMouseEnter={this.props.getBarHeight} 
            ref="WrapComponents"
            >
                {children}
            </div>
        );
    }
}

//The ScrollBar
class ScrollBar extends React.Component<IPScrollBar,ISScrollBar>{
   
    render(){
        //console.log(this.props);
        return (
            <div
            ref="ScrollBar" 
            style={
                {
                ...{//dynamic style , use state instead 
                        height:this.props.height + 'px',
                        marginTop:this.props.top + 'px',
                        right: (this.props.options.float == 'right' ? '0px' : '')
                    },
                ...scrollbarBaseStyle, //scrollbar base style 
                ...this.props.options,//user custom scrollbar style 
                
                }
            } 
           
            >
            </div>
        );
    }
}

class ScrollContainer extends React.Component<IPScrollContainer,ISScrollContainer>{
    ScrollBarRef:React.DetailedHTMLProps<any,any>;
    WrapComponentsRef:React.DetailedHTMLProps<any,any>;
    WrapWrapComponentsRef:React.DetailedHTMLProps<any,any>;
    defaultBarStyleOptions:any = {
        background: '#2c3a2c',
        width: '5px',
        float: 'left'
    }
    minBarHeight = 35    
    defaultDeltaY:number = 50
    state = {
        mousedown:false,
        innerdeltaY:0,
        scrollInnerHeight:0,
        scrollHeight:0,
        height:0,
        top:0
    }
    getBarHeight = () => {
        this.setState((prestate) => {
            let scrollHeight = parseFloat(window.getComputedStyle(this.WrapWrapComponentsRef.refs['WrapWrapComponents']).getPropertyValue("height").replace('px', ""));            
            let scrollInnerHeight = this.WrapComponentsRef.refs['WrapComponents'].scrollHeight;
            let scrollTime = Math.ceil((scrollInnerHeight - scrollHeight) / Math.abs(this.defaultDeltaY));
            let height = Math.max(scrollHeight / (scrollInnerHeight / scrollHeight), this.minBarHeight);            
            let innerdeltaY = (scrollHeight - height) / scrollTime;
            if(scrollHeight >= scrollInnerHeight){
                height = 0;
                innerdeltaY = 0;
            }
            return {
                scrollHeight:scrollHeight,
                scrollInnerHeight:scrollInnerHeight,
                height:height,
                innerdeltaY:innerdeltaY
            }
        },() => {
            if(this.state.scrollHeight < this.state.scrollInnerHeight){
                this.resizeTop();
                this.showBar();
            }
        });
    }
    showBar = () => {
        this.ScrollBarRef.refs['ScrollBar'].style.opacity = 1;
    }
    hideBar = () => {
        if (!this.state.mousedown) {
           // var bar = document.getElementById(this.id2);
           this.ScrollBarRef.refs['ScrollBar'].style.opacity = 0;
        }
    }
    resizeTop = () => {
        //先求出con剩余的值
        let lastHeight = this.state.scrollInnerHeight - this.state.scrollHeight - this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollTop;
        if(lastHeight < 0.1){
            lastHeight = 0;
        }
        let time = Math.abs(Math.ceil(lastHeight / this.defaultDeltaY));
        let top = this.state.scrollHeight - (this.state.height + (time * this.state.innerdeltaY));
        if(this.state.scrollHeight >= this.state.scrollInnerHeight){
            top = 0;
        }
        this.setState({
            top:top  
        })
    };
    listenmouseout = () => {
        this.ScrollBarRef.refs['ScrollBar'].parentNode.addEventListener('mouseout', () => {
           this.hideBar()
        });
    };
    listenwheel = () => {
        this.ScrollBarRef.refs['ScrollBar'].parentNode.addEventListener('wheel', (e:React.WheelEvent<any>) =>  {

            this.getBarHeight();

            this.scrollCon(e.deltaY > 0 ? 1 : -1, 1);
        });
    }
    listenDrag = () => {
        var self = this;
        var y:any;
        var _y;
        function move(e:DocumentEventMap['mousemove']) {

            _y = e.pageY;
            var _delta = _y - y;
            self.scrollCon(_delta > 0 ? 1 : -1, Math.abs(_delta / self.state.innerdeltaY));
            y = _y;

        }
        self.ScrollBarRef.refs['ScrollBar'].addEventListener('mousedown', function(e:DocumentEventMap['mousedown']) {
            //console.log(e);
            self.setState((prestate) => {
                return {
                    mousedown:true
                }
            },() => {
                y = e.pageY;
                document.addEventListener<'mousemove'>('mousemove', move);
                document.addEventListener('mouseup', function(e) {
                    self.setState((prestate) => {
                        return {
                            mousedown:false
                        }
                    },() => {
                        self.hideBar();
                        document.removeEventListener('mousemove', move);
                    });
                });
            })           
        });
    }
    scrollCon = (pos:-1|1, time:number) => {
        //pos：position   1：down  -1：up

        if (!((pos < 0 && this.state.top <= 0) || (this.state.scrollHeight <= this.state.top + this.state.height && pos > 0))) {
            var Top = this.state.top + pos * this.state.innerdeltaY * time;
            var ScrollTop = this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollTop + pos * this.defaultDeltaY* time;
            if (pos < 0) {
                //scroll to up
                this.setState((prestate) => {
                    this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollTop = Math.max(0, ScrollTop);                                        
                    return {
                        top:Math.max(0, Top)
                    }
                },() => {
                })
            } else if (pos > 0) {
                //scroll to down
                this.setState((prestate) => {
                    this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollTop = Math.min(this.state.scrollInnerHeight - this.state.scrollHeight, ScrollTop);                    
                    return {
                        top:Math.min(this.state.scrollHeight - this.state.height, Top)
                    }
                },() => {
                })
            }
        }
        var content:any = {};
        var bar:any = {};
        content.lastScrolled = (this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollHeight - this.WrapComponentsRef.refs['WrapComponents'].scrollTop) + 'px';
        content.hasScrolled = this.WrapWrapComponentsRef.refs['WrapWrapComponents'].scrollTop + 'px';
        bar.hasScrolled = this.state.top + 'px';
        bar.height = this.state.height + 'px';
        bar.lastScrolled = (this.state.scrollHeight - this.state.top - this.state.height) + 'px';
        bar.name = "bar";
        content.name = "content";
        if(this.props.methods){
            this.props.methods.scroll(bar, content);
        }
    }
    componentDidMount(){
        if(this.props.options !== undefined){
            for (let key in this.props.options) {
                if (this.props.options[key] !== undefined && key !== 'deltaY') {
                    this.defaultBarStyleOptions[key] = this.props.options[key]
                } else if(key == 'deltaY' && this.props.options.deltaY){
                    this.defaultDeltaY = this.props.options.deltaY;
                }
            }
        }
        this.getBarHeight();
        this.listenwheel();
        this.listenDrag();
        this.listenmouseout();
    }
    render(){
        const children = this.props.children;
        return (
        <WrapWrapComponents ref={(ref:React.DetailedHTMLProps<any,any>) => {this.WrapWrapComponentsRef = ref}} onMouseEnter={this.showBar} onmouseout={this.hideBar}>
            <WrapComponents ref={(ref:React.DetailedHTMLProps<any,any>) => {this.WrapComponentsRef = ref}} getBarHeight={this.getBarHeight}>
                {children}
            </WrapComponents>
            <ScrollBar top={this.state.top} height={this.state.height} options={this.defaultBarStyleOptions} ref={(ref:React.DetailedHTMLProps<any,any>) => {this.ScrollBarRef = ref}} />
        </WrapWrapComponents>
        )
    }
}

export default ScrollContainer;