var htmlWebpackPlugin = require('html-webpack-plugin')
var path = require('path')
var webpack = require('webpack')
var extractTextPlugin = require('extract-text-webpack-plugin');

function resolve (dir) {
    return path.join(__dirname, dir)
}

module.exports = {
    entry:{
        index:[path.resolve(__dirname,'./server/dev-client.ts'),path.resolve(__dirname,'./src/main.tsx')],
        vendor:['react','react-dom','antd','react-router-dom','axios','querystring'],
        antdcss:['antd/dist/antd.css']
    },
    output:{
        path:path.resolve(__dirname,'./disk'),
        filename:'[name].js',
        chunkFilename:'[name].[hash].js',
        publicPath:'/'
    },
    module:{
        rules:[
            {
                test:/\.tsx?$/,
                use:[
                   'babel-loader',
                    'ts-loader'
                ],
                exclude:/node_modules/
            }, 
            {
                test:/\.(css|less)$/,
                use: extractTextPlugin.extract({
                    fallback: "style-loader",
                    use: "css-loader"
                }) 
            },
            {
                test:/\.(jpg|png|jpeg)$/,
                loader:'url-loader'
            }
        ]
    },
    resolve:{
        extensions:['.js','.ts','.tsx']      
    },
    devtool:"#source-map",
    plugins:[
        new htmlWebpackPlugin({
            inject:true,
            template:'src/index.html',
        }),
        new webpack.DefinePlugin({
            'process.env.NODE_ENV': JSON.stringify('env')
        }),
        new webpack.HotModuleReplacementPlugin(),
        new webpack.NoEmitOnErrorsPlugin(),
        new extractTextPlugin("[name].[hash].css"),
        new webpack.optimize.CommonsChunkPlugin({
            name: "vendor",
            filename: "vendor.js"
        })
        //,
        // new webpack.optimize.UglifyJsPlugin({
        //     compress: {
        //         warnings: false
        //     }
        // })
    ]
    // ,
    // externals: {
    //     "react": "React",
    //     "react-dom": "ReactDOM"
    // }
}